import { csv } from 'd3';
import {
  select,
  json,
  pie,
  arc,
  scaleOrdinal,
  format
} from 'd3';

// Adding colors for the sections in Pie Chart
const sectionColors = [
  		"red",
      "blue",
      "yellow",
      "green",
      "orange",
      "grey",
      "purple",
      "brown",
      "magenta",
      "lime"
		  ];

// Specifying the Data set url
const csvUrl = 'https://gist.githubusercontent.com/Eshwarreddyt/3536edfe10f7bc5203ddd6e19a393e9e/raw/e12702a79a79e8fbf7b478328b04f899673d99ca/top10indiastates.csv';

// Function to load data from the Url
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

// Selecting the SVG from the HTML body
const svg = select("svg");
const width = + svg.attr('width');
const height = + svg.attr('height');

// Formatting axes text
const xAxisTickFormat = number =>
  	format('.4~g')(number);

// Rendering Pie Chart
const render = (data, total) => {
  const pieData = pie().value(d=>d.total_population)(data);
  const colors = scaleOrdinal()
  	.domain(data)
  	.range(sectionColors);
  
  // Object for generating arcs for the sections
  const arcGenerator = d3.arc()
  .innerRadius(0)
  .outerRadius(250);

  // Segments are different partitions in the pie chart
	const segments = arc()
  	.innerRadius(0)
  	.outerRadius(175)
  	.padAngle(0)
  	.padRadius(0);
  
  const sections = svg.append("g")
  	.attr("transform", `translate(250,250)`)
  	.selectAll("path").data(pieData);
  
  sections.enter()
    .append("path")
    .attr("d", segments)
  	.attr("fill", d => colors(d.data.total_population));
  
  // Render text on each section
  sections
    .enter()
    .append('text')
    .text(d => xAxisTickFormat((d.data.total_population/1000000))+"M")
    .attr("transform", function(d) { return "translate(" + arcGenerator.centroid(d) + ")";  })
    .style("text-anchor", "middle")
  	.style('fill', 'black')
    .style("font-size", 12);
  
  // Render legend for the data
  const legends = svg.append("g")
  	.attr("transform", "translate(500,100)")
    .selectAll(".legends").data(pieData);
  
  const legend = legends.enter().append("g").classed(".legends",true)
  	.attr("transform", (d,i)=>{
      return `translate(0,${(i+1)*30})`;
    });
  
  // Formats the legend for total population
  legend.append("rect").attr("width",20).attr("height",20)
  	.attr("fill", d => colors(d.data.total_population));
  
  legend.append("text")
  	.attr("x", 250)
  	.attr("y", 15)
  	.attr("class","legend_value")
  	.text(d=>xAxisTickFormat((d.data.total_population/1000000))+"M");
  
  // Formats the legend for States
  legend.append("text")
  	.attr("x", 25)
  	.attr("y", 15)
  	.attr("class","legend_text")
  	.text(d => d.data.State);
  
}

// Main function
const run = async () => {
  let data = await fetchData()
  let total = 0
  data.forEach(d=>{
    d.total_population = +d.total_population;
    total += d.total_population
  });
	render(data,total);
};
run();

